==> pkg_trackinst-1.2.3.tbz: lib/python2.5/site-packages/PkgTrackInst/PkgDbWriter.py <==
#!/usr/bin/env python
#
# PkgTrackInst.PkgDbWriter - Writes new package db entry based on install log
#
# ----------------------------------------------------------------------------
# "THE BEER-WARE LICENSE" (Revision 42, (c) Poul-Henning Kamp):
# Hye-Shik Chang <perky@FreeBSD.org> wrote this file.  As long as you retain
# this notice you can do whatever you want with this stuff. If we meet some
# day, and you think this stuff is worth it, you can buy me a beer in return.
#
# Hye-Shik Chang
# ----------------------------------------------------------------------------
# $Id: PkgDbWriter.py 644 2006-06-25 09:44:16Z perky $

__pychecker__ = 'no-argsused no-isliteral'

import os, sys
import time, md5
import shutil
from PkgTrackInst import options
from PkgTrackInst.cmd_support import errx, echo_msg
from PkgTrackInst.compat23 import *
from PkgTrackInst.Trackers import InstFile, InstDirectory, TrackRoot
from PkgTrackInst.TerminalSlider import TerminalSlider

CONTENTS_HEADER = """\
@comment PKG_FORMAT_REVISION:%(pkg_format_revision)s
@name %(name)s
@comment ORIGIN:%(origin)s
@cwd %(prefix)s
@comment --- Generated using pkg_trackinst on %(generated_date)s"""


class PkgDbWriter(object):

    def __init__(self, dbinfo):
        self.pkginfo = {
            'comment': dbinfo['Comment'],
            'name': '%s-%s' % (dbinfo['PortName'], dbinfo['PortVersion']),
            'pkg_format_revision': '1.1',
            'origin': dbinfo['Origin'],
            'prefix': dbinfo['Prefix'],
            'description': dbinfo['Description'],
            'generated_date': time.asctime(time.localtime()),
        }

        for attr in ('message', 'install_script', 'deinstall_script'):
            if getattr(options, attr):
                name = getattr(options, attr)
                try:
                    self.pkginfo[attr] = open(name).read()
                except (IOError, OSError):
                    errx("Can't open `%s'." % name)

        self.pkginfo['dbdir'] = os.path.join(options.pkgdb,
                                             self.pkginfo['name'])
        if os.path.exists(self.pkginfo['dbdir']):
            errx("`%s' is already there in pkgdb." % self.pkginfo['name'])

    def write(self):
        os.mkdir(self.pkginfo['dbdir'])
        try:
            self.write_baseinfo()
            self.generate_contents_list(self.pkgfile('+CONTENTS'))
            self.write_scripts()
        except Exception:
            shutil.rmtree(self.pkginfo['dbdir'])
            raise

    def pkgfile(self, name):
        return open(os.path.join(self.pkginfo['dbdir'], name), 'w')

    def write_baseinfo(self):
        print >> self.pkgfile('+COMMENT'), self.pkginfo['comment']
        print >> self.pkgfile('+DESC'), self.pkginfo['description'].strip()

        mtree = open('/etc/mtree/BSD.local.dist').read()
        self.pkgfile('+MTREE_DIRS').write(mtree)

    def write_scripts(self):
        for attr, fname in (('message', '+DISPLAY'),
                            ('install_script', '+INSTALL'),
                            ('deinstall_script', '+DEINSTALL')):
            if attr in self.pkginfo:
                self.pkgfile(fname).write(self.pkginfo[attr])

    def relpath(self, fullpath):
        prefix = self.pkginfo['prefix'].split(os.sep)
        full = fullpath.split(os.sep)
        if full[:len(prefix)] == prefix:
            return os.sep.join(full[len(prefix):])
        else:
            return fullpath

    def generate_contents_list(self, out):
        echo_msg('Generating pkgdb contents list')
        print >> out, CONTENTS_HEADER % self.pkginfo

        total_amount = [0]
        fileents = list()
        def gather_filesize(ent):
            if not isinstance(ent, InstDirectory) and ent.created:
                fileents.append(ent)
                if isinstance(ent, InstFile):
                    total_amount[0] += os.path.getsize(ent.abspath)
        TrackRoot.walk(gather_filesize)

        progress = TerminalSlider(total_amount[0], 0)
        done = 0
        fileents.sort(key=lambda ent: ent.abspath)
        for ent in fileents:
            print >> out, self.relpath(ent.abspath)
            if isinstance(ent, InstFile): # not symlink
                md5dig = md5.new(open(ent.abspath).read()).hexdigest()
                print >> out, '@comment MD5:' + md5dig
                done += os.path.getsize(ent.abspath)
                progress.update(done)
        progress.end()

        dirs = []
        def gen_dirlist(ent):
            if isinstance(ent, InstDirectory) and ent.created:
                dirs.append(ent)
        TrackRoot.walk(gen_dirlist)
        dirs.reverse()

        for dirent in dirs:
            print >> out, '@dirrm ' + self.relpath(dirent.abspath)

    def create_package(self):
        if options.pkgdb != '/var/db/pkg':
            echo_msg("Can't create package when pkgdb is not in standard "
                     "path")
            return

        echo_msg('Building package')
        dirname, basename = os.path.split(self.pkginfo['dbdir'])
        status = os.system('/usr/sbin/pkg_create -b ' + basename)
        if status:
            errx('Failed to create a binary package: error %d' % status)

==> pkg_trackinst-1.2.3_2.tbz: lib/python2.7/site-packages/PkgTrackInst/PkgDbWriter.py <==
#!/usr/bin/env python
#
# PkgTrackInst.PkgDbWriter - Writes new package db entry based on install log
#
# ----------------------------------------------------------------------------
# "THE BEER-WARE LICENSE" (Revision 42, (c) Poul-Henning Kamp):
# Hye-Shik Chang <perky@FreeBSD.org> wrote this file.  As long as you retain
# this notice you can do whatever you want with this stuff. If we meet some
# day, and you think this stuff is worth it, you can buy me a beer in return.
#
# Hye-Shik Chang
# ----------------------------------------------------------------------------
# $Id: PkgDbWriter.py 644 2006-06-25 09:44:16Z perky $

__pychecker__ = 'no-argsused no-isliteral'

import os, sys
import time, md5
import shutil
from PkgTrackInst import options
from PkgTrackInst.cmd_support import errx, echo_msg
from PkgTrackInst.compat23 import *
from PkgTrackInst.Trackers import InstFile, InstDirectory, TrackRoot
from PkgTrackInst.TerminalSlider import TerminalSlider

CONTENTS_HEADER = """\
@comment PKG_FORMAT_REVISION:%(pkg_format_revision)s
@name %(name)s
@comment ORIGIN:%(origin)s
@cwd %(prefix)s
@comment --- Generated using pkg_trackinst on %(generated_date)s"""


class PkgDbWriter(object):

    def __init__(self, dbinfo):
        self.pkginfo = {
            'comment': dbinfo['Comment'],
            'name': '%s-%s' % (dbinfo['PortName'], dbinfo['PortVersion']),
            'pkg_format_revision': '1.1',
            'origin': dbinfo['Origin'],
            'prefix': dbinfo['Prefix'],
            'description': dbinfo['Description'],
            'generated_date': time.asctime(time.localtime()),
        }

        for attr in ('message', 'install_script', 'deinstall_script'):
            if getattr(options, attr):
                name = getattr(options, attr)
                try:
                    self.pkginfo[attr] = open(name).read()
                except (IOError, OSError):
                    errx("Can't open `%s'." % name)

        self.pkginfo['dbdir'] = os.path.join(options.pkgdb,
                                             self.pkginfo['name'])
        if os.path.exists(self.pkginfo['dbdir']):
            errx("`%s' is already there in pkgdb." % self.pkginfo['name'])

    def write(self):
        os.mkdir(self.pkginfo['dbdir'])
        try:
            self.write_baseinfo()
            self.generate_contents_list(self.pkgfile('+CONTENTS'))
            self.write_scripts()
        except Exception:
            shutil.rmtree(self.pkginfo['dbdir'])
            raise

    def pkgfile(self, name):
        return open(os.path.join(self.pkginfo['dbdir'], name), 'w')

    def write_baseinfo(self):
        print >> self.pkgfile('+COMMENT'), self.pkginfo['comment']
        print >> self.pkgfile('+DESC'), self.pkginfo['description'].strip()

        mtree = open('/usr/ports/Templates/BSD.local.dist').read()
        self.pkgfile('+MTREE_DIRS').write(mtree)

    def write_scripts(self):
        for attr, fname in (('message', '+DISPLAY'),
                            ('install_script', '+INSTALL'),
                            ('deinstall_script', '+DEINSTALL')):
            if attr in self.pkginfo:
                self.pkgfile(fname).write(self.pkginfo[attr])

    def relpath(self, fullpath):
        prefix = self.pkginfo['prefix'].split(os.sep)
        full = fullpath.split(os.sep)
        if full[:len(prefix)] == prefix:
            return os.sep.join(full[len(prefix):])
        else:
            return fullpath

    def generate_contents_list(self, out):
        echo_msg('Generating pkgdb contents list')
        print >> out, CONTENTS_HEADER % self.pkginfo

        total_amount = [0]
        fileents = list()
        def gather_filesize(ent):
            if not isinstance(ent, InstDirectory) and ent.created:
                fileents.append(ent)
                if isinstance(ent, InstFile):
                    total_amount[0] += os.path.getsize(ent.abspath)
        TrackRoot.walk(gather_filesize)

        progress = TerminalSlider(total_amount[0], 0)
        done = 0
        fileents.sort(key=lambda ent: ent.abspath)
        for ent in fileents:
            print >> out, self.relpath(ent.abspath)
            if isinstance(ent, InstFile): # not symlink
                md5dig = md5.new(open(ent.abspath).read()).hexdigest()
                print >> out, '@comment MD5:' + md5dig
                done += os.path.getsize(ent.abspath)
                progress.update(done)
        progress.end()

        dirs = []
        def gen_dirlist(ent):
            if isinstance(ent, InstDirectory) and ent.created:
                dirs.append(ent)
        TrackRoot.walk(gen_dirlist)
        dirs.reverse()

        for dirent in dirs:
            print >> out, '@dirrm ' + self.relpath(dirent.abspath)

    def create_package(self):
        if options.pkgdb != '/var/db/pkg':
            echo_msg("Can't create package when pkgdb is not in standard "
                     "path")
            return

        echo_msg('Building package')
        dirname, basename = os.path.split(self.pkginfo['dbdir'])
        status = os.system('/usr/sbin/pkg_create -b ' + basename)
        if status:
            errx('Failed to create a binary package: error %d' % status)
